/*
 * Copyright (C) 2013 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ca.mcmaster.cas.tablularexpression;

import java.util.List;
import java.util.ListIterator;

/**
 *
 * @author Matthew Dawson <matthew@mjdsystems.ca>
 */
public class HierarchcialGridCheckerWalkerGenerator {
    
    private static void HandleGrid(SubHierarchyFetcher grid, HierarchcialGridCheckerGenerator generator) {
        List<HierarchicalCell> cells = grid.getSubHiearchy();
        for (int i = 0; i < cells.size(); ++i) {
            HierarchicalCell cell = cells.get(i);
            
            if (cell.getSubHiearchy().isEmpty()) {
                generator.handleLeafCell(cell);
            } else {
                generator.handleEdgeCell(cell);
            }
        }
        
        for (int i = 0; i < cells.size(); ++i) {
            HierarchicalCell cell = cells.get(i);
            
            if (!cell.getSubHiearchy().isEmpty()) {
                generator.descendIntoGridFromCell(cell);
                HandleGrid(cell, generator);
                generator.ascendFromGrid();
            }
        }
    }
    
    public static String GenerateCheckerFromGrid(HierarchicalGrid grid, HierarchcialGridCheckerGenerator generator) {
        HandleGrid(grid, generator);
        return generator.getFinalString();
    }
}
